
// diStorm64 library sample
// http://ragestorm.net/distorm
// Arkon, 2005

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "distorm.h"

// Link the library into our project.
#pragma comment(lib, "distorm.lib")

int main()
{
	// Buffer to disassemble in 64 bits.
	char buf[] = "\xfe\x45\x00\xfe\x05\x01\x00\x00\x00\x4a\xff\x04\xc5\x55\x00\x00\x00\x41\xff\xc7\x48\xb8\x01\x00\x00\x00\x00\x00\x00\x00\x66\x47\x33\xc9\x67\x4f\xc1\xa4\xc0\x45\x23\x01\x00\x55\x48\x99\xc3";

	// Head node, this will hold the returned list.
	_DINode* head;
	// Holds the result of the decoding.
	_DecodeResult res;
	// Decoded instruction information.
	_DecodedInst* di;

	// Decode the buffer at offset 0x100 in 16 bits.
	res = distorm_decode(&head, 0, buf, sizeof(buf)-1, Decode64Bits);
	if (res == DECRES_INPUTERR) {
		// Null buffer? Decode type not 16/32/64?
		printf("Input error, halting!");
		return -1;
	} else if (res == DECRES_MEMORYERR) {
		printf("Not enough memory!\r\n");
		// There might be some data in the linked list, so scan it anyways.
	}

	// As long as there are instructions in the list to read,
	while (di = ll_get(&head)) {
		// Output them in this format:

		// NOTE: If you want to use the di pointer for later use, you will have to copy the buffer to your own buffer, it will be disposed soon!
		printf("0x%08x (%02x) %-24s '%s%s%s'\r\n", di->offset, di->size, (char*)di->instructionHex.p, (char*)di->mnemonic.p, di->operands.length != 0 ? " " : "", (char*)di->operands.p);
	}

	// Use the following code, if you output only some of the results earlier and have to dispose the remaining nodes in the list.
	// while (ll_get(&head)) ;

	return 0;
}